@@ -3,18 +3,22 @@ combnPrim <- function(x,m,simplify=TRUE){
     x <- seq(x)
   if (length(x) < m)
     stop("Error in combnPrim: n < m\n")
-  NCAND = as.integer(length(x))
-  NSEL  = as.integer(m)
+  NCAND <- length(x)
+  NSEL  <- as.integer(m)
   NSET <- as.integer(choose(NCAND,NSEL))
-  ANS  <- as.integer(rep.int(0, NSET*NSEL))
+  ANS  <- rep.int(0L, NSET*NSEL)
   res <- .C("combnC", NSEL, NCAND, NSET, ANS, DUP=FALSE
             ,PACKAGE="gRbase"
   )[[4]]
-  res <- x[res]
-  dim(res) <- c(NSEL, NSET)  
-  if (!simplify){
+
+  if (simplify){
+    matrix(x[res], nrow=NSEL, ncol=NSET)
+  } else {
+    res <- matrix(x[res], nrow=NSEL, ncol=NSET)
     res <- split(res, col(res))
+    ## FIXME: This is *very* inefficient
+    ## FIXME  - use colmat2list or something like that instead
     names(res) <- NULL
+    res
   }
-  res
 }
